/*--------------------------------------------------------------
 | SAS Global Forum 2016 Sample
 | The GEOCODE Procedure and SAS Visual Analytics   
 |    
 | NAME:  GeocodeVA.sas
 | PURPOSE: Examples of using Proc GEOCODE with the VA Centroid 
 |          data to batch geocode countries, "state" level, 
 |          and "county" level data across the globe (US and
 |          non-US). 
 | PAPER: The GEOCODE Procedure and SAS Visual Analytics 
 | DATE:  16Jan2015
 +-------------------------------------------------------------*/

/**** Location where results is stored ****/
libname geo '.';

/**** location of the VA Centlookup and Attrlookup data ****/
libname VAlookup '<location of ATTRLOOKUP and CENTLOOKUP data sets>';





/****************************************************************
 * MACRO:  To run Proc Geocode twice for VA data.  Double lookup.
 *         The 2 step process first converts the character name to
 *         the ID value.  Then, the ID value is used to lookup the
 *         x,y centroid values and the MAPNAME (for choropleth).
 * ARGUMENTS:
 *       outdata - output data set of final geocoded data 
 *       indata  - input address data set
 *       lookupvar - the attribute lookup data set variable name 
 *                   (&startattr)
 *       addrvar - address data set (&indata) variable name
 *       startattr - Optional.  Defaults to VAlookup.attrlookup. 
 *                   Used to specify an alternate attribute lookup 
 *                   data set.  Allows you to subset or alter search
 *
 * NOTE: The VA lookup data has a variable named 'KEY' that contains
 *       all of the pieces of a location, each separated with a 'bar'.
 *       So, Wake County in NC is: "Wake|North Carolina|United States"
 ****************************************************************/

%macro geocode(outdata, indata, lookupvar, addrvar, startattr);

/*startattr is optional, so set default if not specified */
%if (&startattr = ) %then %let startattr=VAlookup.attrlookup;

/* Make sure the data is properly sorted for the search */
proc sort data=&startattr out=attrlookup; by &lookupvar; run;

/* Add x and y because Geocode expects it */
data attrlookup; set attrlookup; x=0; y=0; run;  

/* First, look in the attribute file with the names of the area*/
/* and find the ID variable for that name                      */
proc geocode method=custom
     data= &indata 
	 lookup=attrlookup
	 out=work.attr
	 lookupvar= &lookupvar
	 addressvar= &addrvar
	 attribute_var= (ID )
	 ;
run;

/* Then use the ID value to find the x,y centroid and the MAPNAME */
proc geocode method=custom
     data=work.attr
	 lookup=VAlookup.centlookup
	 out=&outdata
	 lookupvar= ID
	 addressvar=ID
	 attribute_var=mapname
	 ;
run;
%mend;

/*------------------------Example 1-----------------------------*/
/* 9.4 Lookup COUNTRY NAME                                      */
/* to get the ID, centroid x and y and MAPNAME (for choro)      */
data addr;  /* replace this with your data */
   length InputAddr $199;
   InputAddr="United States"; output;
   InputAddr="Canada"; output;
   InputAddr="Mexico"; output;
   InputAddr="Peru"; output;
   InputAddr="Korea, Republic of"; output;
run;

/*countryname=gecoded results, addr=input addresses,  */
/*key=attr lookup ds var name, key=address ds var name  */
%geocode(geo.countryname,addr,key,InputAddr);
/* In VA, Choose Category: ID->Geography->Country SAS Map ID Values */

/*------------------------Example 2-----------------------------*/
/* 9.4 Lookup the 2 character ISO COUNTRY CODE                  */
/* to get the ID, centroid x and y and MAPNAME (for choro)      */
data addr;  /* replace this with your data */
   length InputAddr $199;
   InputAddr="US"; output;
   InputAddr="AD"; output;
   InputAddr="MX"; output;
   InputAddr="TR"; output;
   InputAddr="KR"; output;
   InputAddr="KP"; output;
run;

/*countryabbrev=gecoded results, addr=input addresses,  */
/*ID=attr lookup ds var name, key=address ds var name  */
/* In this case, we use ID to match the country abbrev */
%geocode(geo.countryabbrev,addr,ID,InputAddr);
/* In VA, Choose Category: ID->Geography->Country SAS Map ID Values */


/*------------------------Example 3-----------------------------*/
/* 9.4 Lookup the US STATE AND COUNTRY NAME                     */
/* to get the ID, centroid x and y and MAPNAME (for choro)      */
data addr;  /* replace this with your data */
   length InputAddr $199;
   InputAddr="North Carolina|United States"; output;
   InputAddr="Virginia|United States"; output;
   InputAddr="Wake|North Carolina|United States"; output;
   InputAddr="Richmond|Virginia|United States"; output;
run;

/*uscounties=gecoded results, addr=input addresses,  */
/*key=attr lookup ds var name, key=address ds var name  */
%geocode(geo.uscounties,addr,key,InputAddr);
/* In VA, Choose Category: ID->Geography->Custom */
/* Because this uses STATE and COUNTY            */


/*------------------------Example 4-----------------------------*/
/* 9.4 Lookup the US STATE NAMES without specifying the country */
/* to get the ID, centroid x and y and MAPNAME (for choro)      */
/* NOTE: Attr data is subset to US only.                        */
data addr;  /* replace this with your data */
   length InputAddr $199;
   InputAddr="North Carolina"; output; 
   InputAddr="Virginia"; output;
   InputAddr="California"; output;
run;

/*subset out US states only only */
data myattr;
   set VAlookup.attrlookup;
   if (isoname="UNITED STATES" and level=1) then output;
run;

/*usstatename=gecoded results, addr=input addresses,  */
/*IDNAME=attr lookup ds var name, key=address ds var name  */
/*myattr=alternate stribute data set */
/* Note that myattr contains a subset for the US and */
/* IDNAME is the variable we look at for state names */
%geocode(geo.usstatename,addr,IDNAME,InputAddr,myattr);
/* In VA, Choose Category: ID->Geography->Subdivision SAS Map ID Values */


/*------------------------Example 5-----------------------------*/
/* 9.4 Lookup the US STATE ABBREVIATIONS                         */
/* to get the ID, centroid x and y and MAPNAME (for choro)      */
/* NOTE: A different Attr file is used for US                   */
data addr;  /* replace this with your data */
   length InputAddr $199;
   InputAddr="NC"; output;
   InputAddr="VA"; output;
   InputAddr="CA"; output;
run;

/* The us_states dataset had a variable for the Abbreviations */
data myattr;
   set mapsgfk.us_states_attr;
   Label ID=ID;
   run;

/*usabbrev=gecoded results, addr=input addresses,  */
/*STATECODE=attr lookup ds var name, key=address ds var name  */
/*myattr=alternate stribute data set */
/* Note that myattr contains a different attribute DS */
/* IDNAME is the variable we look at for state names */
%geocode(geo.usabbrev,addr,STATECODE,InputAddr,myattr);
/* In VA, Choose Category: ID->Geography->Subdivision SAS Map ID Values */

/*------------------------Example 6-----------------------------*/
/* 9.4 Lookup the US COUNTY NAMES without specifying the country*/
/* to get the ID, centroid x and y and MAPNAME (for choro)      */
/* NOTE: Attr data is subset to US only.                        */
data addr;  /* replace this with your data */
   length InputAddr $199;
   InputAddr="Wake"; output;
   InputAddr="Durham"; output;
   InputAddr="Orange"; output;
run;

/*subset out US states only only */
data myattr;
   set VAlookup.attrlookup;
   if (isoname="UNITED STATES" and id2name="North Carolina" and level=2) then output;
run;

/*uscountyname=gecoded results, addr=input addresses,      */
/*IDNAME=attr lookup ds var name, key=address ds var name  */
/*myattr=alternate stribute data set */
/* Note that myattr contains a subset for the US and */
/* IDNAME is the variable we look at for state names */
%geocode(geo.uscountyname,addr,IDNAME,InputAddr,myattr);
/* In VA, Choose Category: ID->Geography->Custom */
/* Because this uses COUNTIES                    */

/*------------------------Example 7-----------------------------*/
/* 9.4 Lookup WORLDWIDE "STATE" AND "COUNTRY" levels            */
/* to get the ID, centroid x and y and MAPNAME (for choro)      */
data addr;  /* replace this with your data */
   length InputAddr $199;
   InputAddr="Afyonkarahisar|Turkey"; output;
   InputAddr="Bayat|Afyonkarahisar|Turkey"; output;
   InputAddr="Adiyaman|Turkey"; output;
   InputAddr="Yuregir|Adana|Turkey"; output;
   InputAddr="Durham|Ontario|Canada"; output;
   InputAddr="Cardiff|United Kingdom"; output;
   InputAddr="Wyre|Lancashire|United Kingdom"; output;
run;

/*nonUScounties=gecoded results, addr=input addresses,  */
/*key=attr lookup ds var name, key=address ds var name  */
%geocode(geo.nonUScounties,addr,key,InputAddr);
/* In VA, Choose Category: ID->Geography->Custom */
/* Because this uses STATE and COUNTY            */




/*------------------------Example 8-----------------------------*/
/* 9.4 Lookup NON-US 'STATE' level  (Turkey)                    */
/* to get the ID, centroid x and y and MAPNAME (for choro)      */
/* NOTE: Attr data set is subset for the country (turkey)       */
data addr;  /* replace this with your data */
   length InputAddr $199;
   InputAddr="Afyonkarahisar"; output;
   InputAddr="Adiyaman"; output;
   InputAddr="Adana"; output;
run;

/*subset out Turkey states only only */
data myattr;
   set VAlookup.attrlookup;
   if (isoname="TURKEY" and level=1) then output;
run;

/*nonusstatename=gecoded results, addr=input addresses,  */
/*IDNAME=attr lookup ds var name, key=address ds var name  */
/*myattr=alternate stribute data set */
/* Note that myattr contains a subset for the TURKEY and */
/* IDNAME is the variable we look at for state names     */
%geocode(geo.nonusstatename,addr,IDNAME,InputAddr,myattr);
/* In VA, Choose Category: ID->Geography->Subdivision SAS Map ID Values */




/*------------------------Example 9-----------------------------*/
/* Check the COUNTY ABBREVIATIONS.                              */
/* This makes sure your data is correct                         */
/* Note that the last abbreviation is wrong */
data addr;  /* replace this with your data */
   length CountryAbbr $2 ;
   input CountryAbbr $ population;
datalines;
US 317466000
AD 76246
MX 118395054
TR 75627384
KR 4294000
KP 24895000
ZZ 12973808
;
run;

%geocode(CKcountryabbrev,addr,ID,CountryAbbr);

data CkBadMatch;
  set CKcountryAbbrev (where= (_MATCHED_ = "None"));
  if _N_=1 then put "WARNING: Country Abbreviations values that do not match:";
  put CountryAbbr=;
run;

/*------------------------Example 10----------------------------*/
/* Check the COUNTRY NAME                                       */
/* This makes sure your data is correct                         */
/* Bolivia is wrong.  The ISO name is "Bolivia, Plurinational State of" */
data addr;  /* replace this with your data */
   length Country $200;
   input Country $1-20 population;
datalines;
United States       317466000
Canada              35295770
Mexico              118395054
Peru                30475144
Korea, Republic of  50219669
Bolivia             10027254
;
run;

/* Subset the lookup data to just the country data*/
data myattr; 
   set VALOOKUP.attrlookup(where= (level=0));  /* country level */
run;

/* notice the last parm overrides the default lookup data*/
%geocode(CKcountry,addr,IDNAME,Country,myattr);

data CkBadMatch;
  set CKcountry (where= (_MATCHED_ = "None"));
  if _N_=1 then put "WARNING: Country Name values that do not match:";
  put Country=;
run;
